var EasyCopyBrowserDetails = {
	/**
	 * Gets the document's title without the title modifier, eg. "The Title - Mozilla Firefox" will only return "The
	 * Title".
	 * 
	 * @returns
	 */
	getTitle : function(theDocument) {
		var formattedTitle = theDocument.title;
		
		if (!EasyCopyPrefsManager.removeLeadingAndTrailingText()) {
			return formattedTitle;
		}
		return EasyCopyBrowserDetails.stripTrailingLeadingText(formattedTitle, EasyCopyPrefsManager.leadingAndTrailingTextToRemove());
	},
	
	stripTrailingLeadingText : function(title, trailingTextItems) {
		var formattedTitle = title;
		for (var i = 0; i < trailingTextItems.length; i++) {
			var trailingTextItem = trailingTextItems[i].replace(/([\|.?*+^$[\]\\(){}-])/g, "\\$1");	// Escape special regex characters in the title
			var trailingTextRegEx = new RegExp("^" + trailingTextItem + "[ ]*|" + "[ ]*" + trailingTextItem + "$");
			if (trailingTextRegEx.test(formattedTitle)) {
				formattedTitle = formattedTitle.replace(trailingTextRegEx, "");
				break;
			}
		}
		
		return formattedTitle;
	},

	getUrl : function(theDocument) {
		return theDocument.defaultView.location.href;
	},
	
	getLinkText : function(theDocument) {
		if (!EasyCopyBrowserDetails.onLink(theDocument)) {
			return "";
		}
		
		return gContextMenu.linkText();
	},
	
	getLinkUrl : function(theDocument) {
		if (!EasyCopyBrowserDetails.onLink(theDocument)) {
			return "";
		}

		return gContextMenu.linkURL;
	},

	documentIsActive : function(theDocument) {
		return theDocument == gBrowser.contentDocument;
	},
	
	onLink : function(theDocument) {
		return (EasyCopyBrowserDetails.documentIsActive(theDocument) 
				&& gContextMenu != null && gContextMenu.onLink);
	},
	
	focusedWindow : function() {
		return document.commandDispatcher.focusedWindow;
	},
	
	getSelection : function(theDocument) {
		if (!EasyCopyBrowserDetails.contentIsSelected(theDocument)) {
			return "";
		}
		
		return theDocument.defaultView.getSelection().toString();
	},
	
	contentIsSelected : function(theDocument) {
		return !theDocument.defaultView.getSelection().isCollapsed;
	},
	
	getSelectedHtmlSource : function(theDocument) {
		var range = theDocument.defaultView.getSelection().getRangeAt(0);
	    var node = theDocument.createElement("span");
	    node.appendChild(range.cloneContents());
	    return node.innerHTML;
	},
	
	getDocumentHtmlSource : function(theDocument) {
		return theDocument.documentElement.innerHTML;
	},
	
	/**
	 * Prompts the user for a value and returns it as a string. If the user pressed Cancel then blanks are returned.
	 */
	userPrompt : function(theDocument) {
		var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
							.getService(Components.interfaces.nsIPromptService);

		var value = {value: EasyCopyBrowserDetails.contextualTitle(theDocument)};	//Default to contextual title to help user out a bit
		var okPressed = promptService.prompt(EasyCopyBrowserDetails.focusedWindow(), "Easy Copy - Notes for [" + EasyCopyBrowserDetails.getTitle(theDocument) + "]", 
			"Enter your own notes to copy to the clipboard:", value, null, {} );

		if (!okPressed) {
			return "";		// Cancel pressed
		}
		return value.value;
	},
	
	contextualTitle : function(theDocument) {
		if (EasyCopyBrowserDetails.contentIsSelected(theDocument)) {
			return EasyCopyBrowserDetails.getSelection(theDocument);
		}
		
		if (EasyCopyBrowserDetails.onLink(theDocument)) {
			return EasyCopyBrowserDetails.getLinkText(theDocument);
		}
		
		return EasyCopyBrowserDetails.getTitle(theDocument);
	},
	
	/**
	 * Returns the correct newline character based on the OS. 
	 */
	newline : function() {
        var platform = navigator.platform.toLowerCase();
        if (platform.indexOf('win') != -1) {
        	return "\r\n";
        } else if (platform.indexOf('mac') != -1) {
        	return "\r";
        } else {
        	return "\n";	//Linux or anything that's unknown.
        }
	},
};