var EasyCopy = {
	clipboard : null,
	variableManager : null,
	
	templates : {},
	defaultKeys : {},

	prefsInternal : null,
	// Keeps track of when templates change to figure out if we should rebuild the context menu the next time it's shown or not. 
	rebuildMenu : false,
	
	init : function() {
		EasyCopyPrefsManager.loadDefaultPrefs();
		
		EasyCopy.defaultKeys[1312486368595] = {modifiers: "accel shift", key: "Y"};	//Copy Title
		EasyCopy.defaultKeys[1312486368599] = {modifiers: "accel shift", key: "U"}; //Copy URL

		document.getElementById("contentAreaContextMenu").addEventListener("popupshowing", EasyCopy.handleContextMenuShowing, false);
		
		EasyCopy.variableManager = new EasyCopyVariableManager();
		EasyCopy.registerVariables();

		EasyCopy.buildTemplatesFromPrefs();
		EasyCopy.buildContextMenu();

		EasyCopy.prefsInternal = Components.classes["@mozilla.org/preferences-service;1"].getService(
			Components.interfaces.nsIPrefService).getBranch("extensions.easycopy.");
		EasyCopy.prefsInternal.QueryInterface(Components.interfaces.nsIPrefBranch2);
		EasyCopy.prefsInternal.addObserver("", EasyCopy, false);
		
		EasyCopy.clipboard = new EasyCopyClipboard();
	},
	
	handleContextMenuShowing : function() {
		if (EasyCopy.rebuildMenu) {
			EasyCopy.buildTemplatesFromPrefs();
			EasyCopy.buildContextMenu();
		}
		
		EasyCopyMenuManager.enableMenuBasedOnContext(document.getElementById("contentAreaContextMenu"), EasyCopy.variableManager);
		
		var submenu = document.getElementById("easycopy-submenu");
		if (submenu) {
			EasyCopyMenuManager.enableMenuBasedOnContext(submenu, EasyCopy.variableManager);
		}
		
		EasyCopyMenuManager.showSubmenusBasedOnPrefs();
		EasyCopyMenuManager.hideSubMenusIfEmpty();
	},
	
	observe : function(subject, topic, data) {
		if (topic != "nsPref:changed") {
			return;
		}
		
		EasyCopy.rebuildMenu = true;
	},

	buildTemplatesFromPrefs : function() {
		EasyCopy.templates = EasyCopyPrefsManager.templatesFromPrefs();
	},
	
	buildContextMenu : function() {
		EasyCopy.rebuildMenu = false;

		EasyCopyMenuManager.clearContextMenuItems();
		EasyCopyMenuManager.loadContextMenuItems();

		EasyCopyMenuManager.clearSubMenuItems();
		EasyCopyMenuManager.loadSubMenuItems();
		
		EasyCopyMenuManager.clearAllTabsMenuItems();
		EasyCopyMenuManager.loadAllTabsMenuItems();
		
		EasyCopyKeyManager.clearKeys();
		EasyCopyKeyManager.loadKeys();
	},
	
	copyTemplate : function(template) {
		var resolvedTemplate = EasyCopy.variableManager.resolveTemplate(template, gBrowser.contentDocument);
		EasyCopy.copyToClipboardBasedOnTemplate(resolvedTemplate, template);
	},
	
	copyTemplateAllTabs : function(template) {
		var allResolvedTemplates = [];
		
		var allTabs = EasyCopyWindowManager.allTabs(EasyCopyPrefsManager.includeAllWindows());
		for (var i = 0; i < allTabs.length; i++) {
			var tab = allTabs[i];
			var resolvedTemplate = EasyCopy.variableManager.resolveTemplate(template, tab.contentDocument);

			if (template.type == EasyCopyTemplate.HTML_TYPE) {
				resolvedTemplate = EasyCopy.addMarkupToTemplate(resolvedTemplate, EasyCopyPrefsManager.htmlAddedMarkup());
			} else {
				resolvedTemplate = EasyCopy.addMarkupToTemplate(resolvedTemplate, EasyCopyPrefsManager.textAddedMarkup());
			}
			
			allResolvedTemplates.push(resolvedTemplate);
		} 

		var lineSeparator = EasyCopyBrowserDetails.newline();
		if (template.type == EasyCopyTemplate.HTML_TYPE) {
			lineSeparator = EasyCopyPrefsManager.htmlNewlineTag() + lineSeparator;
		}
		
		EasyCopy.copyToClipboardBasedOnTemplate(allResolvedTemplates.join(lineSeparator), template);
	},

	addMarkupToTemplate : function(resolvedTemplate, markup) {
		if (markup == "") {
			return resolvedTemplate;
		}
		
		if (markup.indexOf("%template%") != -1) {
			resolvedTemplate = markup.replace("%template%", resolvedTemplate, "ig");
		} else {
			resolvedTemplate = markup + resolvedTemplate;
		}
		
		return resolvedTemplate;
	},
	
	copyToClipboardBasedOnTemplate : function(textToCopy, template) {
		if (template.type == EasyCopyTemplate.HTML_TYPE) {
			EasyCopy.clipboard.copy(textToCopy, EasyCopyClipboard.HTML);
		} else {
			EasyCopy.clipboard.copy(textToCopy, EasyCopyClipboard.TEXT);
		}
	},
	
	registerVariables : function() {
		EasyCopy.variableManager.registerVariable("title", new EasyCopyTitleVariableResolver());
		EasyCopy.variableManager.registerVariable("url", new EasyCopyUrlVariableResolver());
		EasyCopy.variableManager.registerVariable("page_title", new EasyCopyPageTitleVariableResolver());
		EasyCopy.variableManager.registerVariable("page_url", new EasyCopyPageUrlVariableResolver());
		EasyCopy.variableManager.registerVariable("selection", new EasyCopySelectionVariableResolver());
		EasyCopy.variableManager.registerVariable("link_title", new EasyCopyLinkTextVariableResolver());
		EasyCopy.variableManager.registerVariable("link_url", new EasyCopyLinkUrlVariableResolver());
		EasyCopy.variableManager.registerVariable("date", new EasyCopyTodayDateVariableResolver());
		EasyCopy.variableManager.registerVariable("time", new EasyCopyTodayTimeVariableResolver());
		EasyCopy.variableManager.registerVariable("date_time", new EasyCopyTodayDateTimeVariableResolver());
		EasyCopy.variableManager.registerVariable("html_source", new EasyCopyHtmlSourceVariableResolver());
		EasyCopy.variableManager.registerVariable("prompt", new EasyCopyPromptVariableResolver());
	}
	
};

window.addEventListener("load", EasyCopy.init, false);