var EasyCopyMenuManager = {
	TEXT_ICON : "chrome://easycopy/skin/text-type.png",
	HTML_ICON : "chrome://easycopy/skin/html-type.png",
	
	MAIN_MENU : "easycopy-main-menu",
	SUBMENU : "easycopy-submenu",
	ALL_TABS_SUBMENU : "easycopy-alltabs-submenu",
	
	buildMenuItemFromTemplate : function(template, idPrefix, namePrefix, commandToExecute) {
		var menuItem = document.createElement("menuitem");
		menuItem.setAttribute("label", namePrefix + template.name);
		menuItem.setAttribute("id", idPrefix + template.id);
		menuItem.setAttribute("class", "menuitem-iconic");
		menuItem.easyCopyTemplate = template;
		
		var icon = EasyCopyMenuManager.TEXT_ICON;
		if (template.type == EasyCopyTemplate.HTML_TYPE) {
			icon = EasyCopyMenuManager.HTML_ICON;
		}
		menuItem.setAttribute("image", icon);
		
		menuItem.addEventListener("command", commandToExecute, false);

		return menuItem;
	},

	callCopyTemplate : function(event) {
		EasyCopy.copyTemplate(this.easyCopyTemplate);
	},
	
	callCopyTemplateAllTabs : function(event) {
		EasyCopy.copyTemplateAllTabs(this.easyCopyTemplate);
	},
	
	buildAllTabsMenuItems : function(templates) {
		var menuItems = [];
		EasyCopyTemplate.forEach(templates, function(template) {
			var menuItem = EasyCopyMenuManager.buildMenuItemFromTemplate(template, EasyCopyMenuManager.ALL_TABS_SUBMENU + "-template-", "",
				EasyCopyMenuManager.callCopyTemplateAllTabs);
			menuItem.setAttribute("key", "easycopy-alltabs-key-" + template.id);

			menuItems.push(menuItem);
		});
		
		menuItems.push(document.createElement("menuseparator"));
		menuItems.push(EasyCopyMenuManager.buildHideThisMenuItem());
		
		return menuItems;
	},
	
	buildHideThisMenuItem : function() {
		var menuItem = document.createElement("menuitem");
		menuItem.setAttribute("label", "Hide This Menu");
		menuItem.setAttribute("id", "easycopy-alltabs-hidethismenu");
		menuItem.setAttribute("class", "menuitem-iconic");
		
		menuItem.addEventListener("command", function(event) {
			EasyCopyPrefsManager.setShowAllTabsSubmenu(false);
		}, false);
		
		return menuItem;
	},
	
	buildSubMenuItems : function(templates) {
		var menuItems = [];
		EasyCopyTemplate.forEach(templates, function(template) {
			var menuItem = EasyCopyMenuManager.buildMenuItemFromTemplate(template, EasyCopyMenuManager.SUBMENU + "-template-", "",
				EasyCopyMenuManager.callCopyTemplate);
			menuItem.setAttribute("key", "easycopy-key-" + template.id);
			
			menuItems.push(menuItem);
		});
		return menuItems;
	},
	
	buildContextMenuItems : function(templates) {
		var menuItems = [];
		EasyCopyTemplate.forEach(templates, function(template) {
			if (!template.showInMainMenu) {
				return;
			}

			var menuItem = EasyCopyMenuManager.buildMenuItemFromTemplate(template, EasyCopyMenuManager.MAIN_MENU + "-template-", "Copy ",
				EasyCopyMenuManager.callCopyTemplate);
			menuItems.push(menuItem);
		});
		return menuItems;
	},
	
	loadContextMenuItems : function() {
		var contextMenuItems = EasyCopyMenuManager.buildContextMenuItems(EasyCopy.templates);
		var contextMenu = document.getElementById("contentAreaContextMenu");
		for ( var i = 0; i < contextMenuItems.length; i++) {
			contextMenu.insertBefore(contextMenuItems[i], document.getElementById(EasyCopyMenuManager.MAIN_MENU));
		}
	},
	
	loadSubMenuItems : function() {
		var menuItems = EasyCopyMenuManager.buildSubMenuItems(EasyCopy.templates);
		var menu = document.getElementById(EasyCopyMenuManager.SUBMENU);
		for ( var i = 0; i < menuItems.length; i++) {
			menu.appendChild(menuItems[i]);
		}
	},

	loadAllTabsMenuItems : function() {
		var menuItems = EasyCopyMenuManager.buildAllTabsMenuItems(EasyCopy.templates);
		var menu = document.getElementById(EasyCopyMenuManager.ALL_TABS_SUBMENU);
		for ( var i = 0; i < menuItems.length; i++) {
			menu.appendChild(menuItems[i]);
		}
	},
	
	hideSubMenuIfEmpty : function(menuId) {
		var menu = document.getElementById(menuId);
		
		//If it's hidden already, then we just leave it as is.
		if (menu.parentNode.getAttribute("hidden") == "true") {
			return;
		}
		menu.parentNode.setAttribute("hidden", menu.children.length === 0);
	},
	
	hideSubMenusIfEmpty : function() {
		EasyCopyMenuManager.hideSubMenuIfEmpty(EasyCopyMenuManager.SUBMENU);
		EasyCopyMenuManager.hideSubMenuIfEmpty(EasyCopyMenuManager.ALL_TABS_SUBMENU);
	},
	
	showSubmenuBasedOnPref : function(menuId, show) {
		var menu = document.getElementById(menuId);
		menu.parentNode.setAttribute("hidden", !show);
	},
	
	showSubmenusBasedOnPrefs : function() {
		EasyCopyMenuManager.showSubmenuBasedOnPref(EasyCopyMenuManager.SUBMENU, EasyCopyPrefsManager.showSubmenu());
		EasyCopyMenuManager.showSubmenuBasedOnPref(EasyCopyMenuManager.ALL_TABS_SUBMENU, EasyCopyPrefsManager.showAllTabsSubmenu());
	},
	
	clearContextMenuItems : function() {
		var contextMenu = document.getElementById("contentAreaContextMenu");
		var i = contextMenu.children.length;

		while (i--) {
		    var menuItem = contextMenu.children[i];

		    if (menuItem.getAttribute("id").indexOf(EasyCopyMenuManager.MAIN_MENU + "-template-") == 0) {
		    	contextMenu.removeChild(menuItem);
		    }
		}		
	},
	
	clearSubMenuItems : function() {
		var menu = document.getElementById(EasyCopyMenuManager.SUBMENU);
		
		while (menu.firstChild) {
			menu.removeChild(menu.firstChild);
		}
	},
	
	clearAllTabsMenuItems : function() {
		var menu = document.getElementById(EasyCopyMenuManager.ALL_TABS_SUBMENU);
		
		while (menu.firstChild) {
			menu.removeChild(menu.firstChild);
		}
	},
	
	enableMenuBasedOnContext : function(menu, variableManager) {
		var templateItemRegEx = /easycopy-.*-template/;
		for (var i = 0; i < menu.children.length; i++) {
			var menuItem = menu.children[i];
			if (!templateItemRegEx.test(menuItem.getAttribute("id"))) {
				continue;
			}
			
			menuItem.setAttribute("disabled", !variableManager.templateAppliesToContext(menuItem.easyCopyTemplate, gBrowser.contentDocument));
		}
	}

};