var EasyCopyPrefsManager = {
	browserPrefs : Components.classes["@mozilla.org/preferences-service;1"].getService(
				Components.interfaces.nsIPrefService).getBranch("browser."),
	preferences : Components.classes["@mozilla.org/preferences-service;1"].getService(
					Components.interfaces.nsIPrefService).getBranch("extensions.easycopy."),

	templatePrefs : Components.classes["@mozilla.org/preferences-service;1"]
		.getService(Components.interfaces.nsIPrefService).getBranch("extensions.easycopy.templates."),
	
	loadDefaultPrefs : function() {
		// For now if the version's set then prefs have already been loaded. Later versions will test a bit more sophisticated.
		if (EasyCopyPrefsManager.lastPrefsVersion() != "") {
			return;
		}
		
		//Hardcoded for now - will update later with extension's version.
		EasyCopyPrefsManager.setLastPrefsVersion("2.0.1");
		EasyCopyPrefsManager.loadTemplatePrefs();
	},
	
	loadTemplatePrefs : function() {
		EasyCopyPrefsManager.loadPref("templates.1312486368595", "<template><id>1312486368595</id><template>&lt;a href=\"%url%\"&gt;%title%&lt;/a&gt;</template><name>Title Link</name><type>html</type><formatter>html</formatter><order>0</order><enabled>true</enabled><showInMainMenu>true</showInMainMenu></template>");
		EasyCopyPrefsManager.loadPref("templates.1312486368596", "<template><id>1312486368596</id><template>[url=%url%]%title%[/url]</template><name>BBCode (Forum) Link</name><type>text</type><formatter>none</formatter><order>4</order><enabled>true</enabled><showInMainMenu>false</showInMainMenu></template>");
		EasyCopyPrefsManager.loadPref("templates.1312486368598", "<template><id>1312486368598</id><template>%title%</template><name>Title</name><type>text</type><formatter>none</formatter><order>2</order><enabled>true</enabled><showInMainMenu>false</showInMainMenu></template>");
		EasyCopyPrefsManager.loadPref("templates.1312486368599", "<template><id>1312486368599</id><template>%url%</template><name>URL</name><type>text</type><formatter>none</formatter><order>1</order><enabled>true</enabled><showInMainMenu>false</showInMainMenu></template>");
		EasyCopyPrefsManager.loadPref("templates.1312486368600", "<template><id>1312486368600</id><template>[%url% %title%]</template><name>Wiki Link</name><type>text</type><formatter>none</formatter><order>5</order><enabled>true</enabled><showInMainMenu>false</showInMainMenu></template>");
		EasyCopyPrefsManager.loadPref("templates.1312486368601", "<template><id>1312486368601</id><template>%html_source%</template><name>HTML Source</name><type>html</type><formatter>none</formatter><order>7</order><enabled>true</enabled><showInMainMenu>false</showInMainMenu></template>");
		EasyCopyPrefsManager.loadPref("templates.1326398956490", "<template><id>1326398956490</id><template>&lt;a href=\"%page_url%\"&gt;%page_title%&lt;/a&gt; - %selection%</template><name>Title Link &amp; Selection</name><type>html</type><formatter>html</formatter><order>3</order><enabled>true</enabled><showInMainMenu>false</showInMainMenu></template>");
	},
	
	loadPref : function(key, value) {
		//Only load the pref if it's not already been loaded before (eg. by prev version of extension) to avoid defaults overwriting existing changes.
		if (EasyCopyPrefsManager.preferences.prefHasUserValue(key)) {
			return;
		}
		EasyCopyPrefsManager.writeCharPref(key, value);
	},
	
	setLastPrefsVersion : function(value) {
		return EasyCopyPrefsManager.preferences.setCharPref("lastPrefsVersion", value);
	},
		
	lastPrefsVersion : function() {
		return EasyCopyPrefsManager.preferences.getCharPref("lastPrefsVersion");
	},
	
	templatesFromPrefs : function() {
		var templates = [];
		var templateKeys = EasyCopyPrefsManager.templatePrefs.getChildList("", {});
		for (var templateKey in templateKeys) {
			try {
				var template = EasyCopyPrefsManager.templateFromPref(templateKeys[templateKey]);
				templates.push(template);
			} catch (e) {
				// Ignore any problem with reading the preferences and just proceed to the next one.
				// Don't want to fall over just because one of the pref's might be wrong. 
				Components.utils.reportError(e);
			}
		}

		return EasyCopyPrefsManager.orderTemplates(templates);
	},
	
	orderTemplates : function(templates) {
		templates.sort(function(template1, template2) {
			if (template1.order == template2.order) {
				return template1.name.toLowerCase().localeCompare(template2.name.toLowerCase());
			} else if (template1.order == -1) {
				return 1;
			} else if (template2.order == -1) {
				return -1;
			}
			
			return template1.order - template2.order;
		});
		
		return templates;
	},
	
	templateFromPref : function(templateKey) {
		var pref = EasyCopyPrefsManager.decodeForReading(EasyCopyPrefsManager.templatePrefs.getCharPref(templateKey));

		var xmlParser = new DOMParser();
		var xml = xmlParser.parseFromString(pref, "text/xml").firstChild;

		var template = new EasyCopyTemplate();
		template.id = xml.getElementsByTagName("id")[0].textContent;;
		template.name = xml.getElementsByTagName("name")[0].textContent;
		template.template = xml.getElementsByTagName("template")[0].textContent;
		template.enabled = EasyCopyPrefsManager.stringToBool(EasyCopyPrefsManager.defaultIfTagNotFound(xml, "enabled", "true"));
		template.order = EasyCopyPrefsManager.defaultIfTagNotFound(xml, "order", -1);
		template.type = EasyCopyPrefsManager.defaultIfTagNotFound(xml, "type", EasyCopyTemplate.TEXT_TYPE);
		template.formatter = EasyCopyPrefsManager.defaultIfTagNotFound(xml, "formatter", EasyCopyTemplate.NO_FORMATTER);
		template.showInMainMenu = EasyCopyPrefsManager.stringToBool(EasyCopyPrefsManager.defaultIfTagNotFound(xml, "showInMainMenu", "false"));

		return template;
	},
	
	stringToBool : function(string) {
		return string.toLowerCase() === "true";
	},
	
	defaultIfTagNotFound : function(xml, tagName, defaultValue) {
		var tags = xml.getElementsByTagName(tagName);
		if (tags.length === 0) {
			if (typeof defaultValue === "undefined") {
				return "";
			} else {
				return defaultValue;
			}
		}
		
		return tags[0].textContent;
	},
	
	/**
	 * Writes out the array of <code>templates</code> passed in to the preferences in XML format.
	 * @param templates
	 */
	writePrefsFromTemplate : function(templates) {
		for (var i = 0; i < templates.length; i++) {
			var template = templates[i];
			
			var xml = EasyCopyPrefsManager.buildXmlFromTemplate(template);

			var templatePref = new XMLSerializer().serializeToString(xml);
			templatePref = EasyCopyPrefsManager.encodeForWriting(templatePref);
			EasyCopyPrefsManager.templatePrefs.setCharPref(template.id, templatePref);
		}
	},

	/**
	 * Returns an XML Document based on the <code>template</code> passed in. The tag names are taken directly from the
	 * name of the properties in the template. Produces XML of the form <template><name>Name</name>...</template>
	 */
	buildXmlFromTemplate : function(template) {
		var xml = document.implementation.createDocument("", "", null);
		var templateXml = xml.createElement("template");
		for (var tag in template) {
			if (!template.hasOwnProperty(tag)) {
				continue;
			}
			
			var tagXml = xml.createElement(tag);
			tagXml.appendChild(xml.createTextNode(template[tag]));
			templateXml.appendChild(tagXml);
		}
		xml.appendChild(templateXml);
		
		return xml;
	},
	
	/**
	 * Returns an array of text strings to remove from the start and end of the title. Only non-blank strings are
	 * included and if there aren't any valid strings then an empty array is returned.
	 */
	leadingAndTrailingTextToRemove : function() {
		var trailingText = EasyCopyPrefsManager.readCharPref("leadingAndTrailingTextToRemove");

		var items = trailingText.split("\n");
		var validItems = [];
		for (var i = 0; i < items.length; i ++) {
			var item = items[i];
			if (item.trim() !== "")	 {
				validItems.push(item);
			}
		}
		return validItems;
	},
	
	encodeForWriting : function(value) {
		// https://developer.mozilla.org/en-US/docs/Firefox_addons_developer_guide/Using_XPCOM%E2%80%94Implementing_advanced_processes
		return unescape(encodeURIComponent(value));
	},
	
	decodeForReading : function(value) {
		// getCharPref returns UTF-8 characters. Convert to UTF-16 using technique described in 
		// https://developer.mozilla.org/en-US/docs/Firefox_addons_developer_guide/Using_XPCOM%E2%80%94Implementing_advanced_processes
		return decodeURIComponent(escape(value));
	},
	
	readCharPref : function(prefKey) {
		return EasyCopyPrefsManager.decodeForReading(EasyCopyPrefsManager.preferences.getCharPref(prefKey));
	},
	
	writeCharPref : function(key, value) {
		return EasyCopyPrefsManager.preferences.setCharPref(key, EasyCopyPrefsManager.encodeForWriting(value));
	},
	
	removeLeadingAndTrailingText : function() {
		return EasyCopyPrefsManager.preferences.getBoolPref("removeLeadingAndTrailingText");
	},
	
	instantApply : function() {
		return EasyCopyPrefsManager.browserPrefs.getBoolPref("preferences.instantApply");
	},
	
	includeWindows : function() {
		return EasyCopyPrefsManager.preferences.getCharPref("allTabs.includeWindows");
	},
	
	includeAllWindows : function() {
		return (EasyCopyPrefsManager.includeWindows() == "all");
	},
	
	textAddedMarkup : function() {
		return EasyCopyPrefsManager.readCharPref("allTabs.textAddedMarkup");
	},
	
	htmlAddedMarkup : function() {
		return EasyCopyPrefsManager.readCharPref("allTabs.htmlAddedMarkup");
	},
	
	htmlNewlineTag : function() {
		return EasyCopyPrefsManager.readCharPref("allTabs.htmlNewlineTag");
	},
	
	showSubmenu : function() {
		return EasyCopyPrefsManager.preferences.getBoolPref("showSubmenu");
	},
	
	showAllTabsSubmenu : function() {
		return EasyCopyPrefsManager.preferences.getBoolPref("allTabs.showAllTabsSubmenu");
	},
	
	setShowAllTabsSubmenu : function(show) {
		EasyCopyPrefsManager.preferences.setBoolPref("allTabs.showAllTabsSubmenu", show);
	},
	
	deleteTemplate : function(template) {
		//Since all templates are user-set (even default ones) clearing one actually deletes it. 
		//This also triggers a an observer notification whereas deleteBranch doesn't.
		EasyCopyPrefsManager.templatePrefs.clearUserPref(template.id);
	}
};