var EasyCopyPrefsUI = {
	templates : {},
	templatesTree : null,
	deletedTemplates : [],
	
	initPrefsDialog : function() {
		try {
			EasyCopyPrefsUI.templates = EasyCopyPrefsManager.templatesFromPrefs();
			EasyCopyPrefsUI.templatesTree = document.getElementById("easycopy-templates");
			
			EasyCopyPrefsUI.buildTreeFromTemplates();

			EasyCopyPrefsUI.handleTreeSelectionChange();
			EasyCopyPrefsUI.templatesTree.view.selection.select(0);  //Select first item in tree

			EasyCopyPrefsUI.hookTreeToDetailFields();
			EasyCopyPrefsUI.enableTemplateDetailsTextFields();
		} catch (e) {
			Components.utils.reportError(e);
		}	
	},
	

	/**
	 * Enables text fields in the template details area to work around broadcaster/observer problem on textboxes. They don't
	 * behave correctly when the broadcaster's disabled attribute is set to false (eg. can get focus). See
	 * https://bugzilla.mozilla.org/show_bug.cgi?id=174492
	 */
	enableTemplateDetailsTextFields : function() {
		document.getElementById("template-name-text").disabled = false;
		document.getElementById("template-template-text").disabled = false;
	}, 
	
	handleTreeSelectionChange : function() {
		EasyCopyPrefsUI.templatesTree.addEventListener("select", function(event) {
			EasyCopyPrefsUI.initTreeButtons();
			EasyCopyPrefsUI.initTemplateDetails();
		}, false);
	},
	
	hookTreeToDetailFields : function() {
		EasyCopyPrefsUI.addChangeListenerForDetailsText("template-name-text", "name-column", function(template, value) {template.name = value;});
		EasyCopyPrefsUI.addChangeListenerForDetailsText("template-template-text", "template-column", function(template, value) {template.template = value;});
		EasyCopyPrefsUI.addChangeListenerForDetailsCheckbox("template-show-in-main-menu-check", "show-in-main-menu-column", function(template, value) {template.showInMainMenu = value;});
		EasyCopyPrefsUI.addChangeListenerForDetailsCheckbox("template-enabled-check", "enabled-column", function(template, value) {template.enabled = value;});
		EasyCopyPrefsUI.addChangeListenerForDetailsDropdown("template-type-menu", "type-column", function(template, value) {template.type = value;});
		EasyCopyPrefsUI.addChangeListenerForDetailsCheckbox("template-html-formatting-check", "html-formatting-column", function(template, value) {template.formatter = (value ? EasyCopyTemplate.HTML_FORMATTER : EasyCopyTemplate.NO_FORMATTER);});

		//Title clean formatting checkboxes		
		document.getElementById("remove-leading-trailing-text-check").addEventListener("click", function(event) {
			document.getElementById("title-text-to-clean-text").disabled = !EasyCopyDomUtils.getBoolValue("remove-leading-trailing-text-check");
		}, false);
		document.getElementById("title-text-to-clean-text").disabled = !EasyCopyDomUtils.getBoolValue("remove-leading-trailing-text-check");
	},
	


	/**
	 * Adds a listener to the textbox with ID <code>textId</code> so when it changes it should update the tree column
	 * <code>colId</code> and modify the selected item's template by calling <code>modifyTemplate</code>.
	 */
	addChangeListenerForDetailsText : function(textId, colId, modifyTemplate) {
		document.getElementById(textId).addEventListener("input", function(event) {
			var value = EasyCopyDomUtils.getTextValue(textId);
			EasyCopyPrefsUI.templatesTree.view.setCellText(EasyCopyPrefsUI.selectedIndex(),
				EasyCopyDomUtils.getColumn(colId, EasyCopyPrefsUI.templatesTree),
				value);
			
			modifyTemplate(EasyCopyPrefsUI.selectedItem().easyCopyTemplate, value);
		}, false);
	},
	
	/**
	 * Adds a listener to the checkbox with ID <code>checkId</code> so when it changes it should update the tree column
	 * <code>colId</code> and modify the selected item's template by calling <code>modifyTemplate</code>.
	 */
	addChangeListenerForDetailsCheckbox : function(checkId, colId, modifyTemplate) {
		document.getElementById(checkId).addEventListener("click", function(event) {
			var value = EasyCopyDomUtils.getBoolValue(checkId);
			EasyCopyPrefsUI.templatesTree.view.setCellValue(EasyCopyPrefsUI.selectedIndex(),
				EasyCopyDomUtils.getColumn(colId, EasyCopyPrefsUI.templatesTree),
				value);
			
			modifyTemplate(EasyCopyPrefsUI.selectedItem().easyCopyTemplate, value);
		}, false);
	},
	
	/**
	 * Adds a listener to the dropdown with ID <code>dropdownId</code> so when it changes it should update the tree column
	 * <code>colId</code> and modify the selected item's template by calling <code>modifyTemplate</code>.
	 */
	addChangeListenerForDetailsDropdown : function(dropdownId, colId, modifyTemplate) {
		document.getElementById(dropdownId).addEventListener("command", function(event) {
			var value = document.getElementById(dropdownId).getAttribute("label");
			EasyCopyPrefsUI.templatesTree.view.setCellText(EasyCopyPrefsUI.selectedIndex(),
				EasyCopyDomUtils.getColumn(colId, EasyCopyPrefsUI.templatesTree),
				value);
			
			modifyTemplate(EasyCopyPrefsUI.selectedItem().easyCopyTemplate, value);
		}, false);
	},
	
	buildTreeFromTemplates : function() {
		var treeChildren = EasyCopyPrefsUI.templatesTree.getElementsByTagName("treechildren")[0];

		for (var i = 0; i < EasyCopyPrefsUI.templates.length; i++) {
			try {
				var template = EasyCopyPrefsUI.templates[i];

				treeChildren.appendChild(EasyCopyPrefsUI.buildTemplateItem(template));
			} catch (e) {
				//Ignore error since there might only be something wrong with one of the templates.
				Components.utils.reportError(e);
			}	
		}
	},

	buildTemplateItem : function(template) {
		var item = document.createElement("treeitem");
		item.setAttribute("id", "treeitem-" + template.id);
		var row = document.createElement("treerow");
		row.setAttribute("id", "treerow-" + template.id);
		
		row.appendChild(EasyCopyPrefsUI.createTextCell(template.name));
		row.appendChild(EasyCopyPrefsUI.createTextCell(template.template));
		row.appendChild(EasyCopyDomUtils.createCheckCell(template.enabled));
		row.appendChild(EasyCopyDomUtils.createCheckCell(template.showInMainMenu));
		row.appendChild(EasyCopyPrefsUI.createTextCell(template.type));
		
		item.appendChild(row);
		item.easyCopyTemplate = template;

		return item;
	},
	
	createTextCell : function(label, id) {
		var cell = document.createElement("treecell");
		cell.setAttribute("label", label);
		if (typeof id !== "undefined") {
			cell.setAttribute("id", id);
		}
		
		return cell;
	},
	
	initTreeButtons : function() {
		var selectedItem = EasyCopyPrefsUI.selectedItem();
		document.getElementById("move-up-button").setAttribute("disabled", selectedItem == null || selectedItem.previousSibling === null);
		document.getElementById("move-down-button").setAttribute("disabled", selectedItem == null || selectedItem.nextSibling === null);
		document.getElementById("delete-button").setAttribute("disabled", selectedItem == null);
	},
	
	initTemplateDetails : function() {
		var selectedIndex = EasyCopyPrefsUI.selectedIndex();
		var template = EasyCopyPrefsUI.selectedItem().easyCopyTemplate;
		EasyCopyDomUtils.setTextValue("template-name-text", template.name);
		EasyCopyDomUtils.setTextValue("template-template-text", template.template);
		EasyCopyDomUtils.setBoolValue("template-enabled-check", template.enabled);
		EasyCopyDomUtils.setBoolValue("template-show-in-main-menu-check", template.showInMainMenu);
		EasyCopyDomUtils.setBoolValue("template-html-formatting-check", template.formatter == EasyCopyTemplate.HTML_FORMATTER ? true : false);

		var type = template.type;
		var typeId = "text-type-item";
		if (type == EasyCopyTemplate.HTML_TYPE) {
			typeId = "html-type-item";
		}
		document.getElementById("template-type-menu").selectedItem = document.getElementById(typeId);
	},
	
	/**
	 * Enables/disables the template details area based on whether there are any templates in the tree. Also clears the fields if there aren't any.
	 * 
	 * Currently called when New/Delete is pressed because I couldn't find a good event to listen to tree changes.
	 */
	handleNumRowsChanged : function() {
		if (EasyCopyPrefsUI.templatesTree.view.rowCount > 0) {
			document.getElementById("template-details-enabled-brdcst").setAttribute("disabled", false);
			EasyCopyPrefsUI.enableTemplateDetailsTextFields();
			return;
		}

		document.getElementById("template-details-enabled-brdcst").setAttribute("disabled", true);
		
		//Clear out all template details fields
		EasyCopyDomUtils.setTextValue("template-name-text", "");
		EasyCopyDomUtils.setTextValue("template-template-text", "");
		EasyCopyDomUtils.setTextValue("template-type-menu", "");
		EasyCopyDomUtils.setBoolValue("template-enabled-check", false);
		EasyCopyDomUtils.setBoolValue("template-show-in-main-menu-check", false);
	},
	
	apply : function() {
		EasyCopyPrefsUI.saveTemplatesToPrefs();
		EasyCopyPrefsUI.deleteTemplatesFromPrefs();
	},
	
	/**
	 * Handles the case when the user cancels the dialog (ie. closes it without clicking Ok) and it's an instantApply
	 * environment (eg. non-Windows). If it's instantApply, then it's assumed that the user wants to apply the settings
	 * so an apply() is issued to update templates. If not instantApply then nothing is done.
	 * 
	 * A way to work around wanting to use prefwindow and get its benefits but not use dialog. See
	 * https://groups.google.com/forum/#!topic/mozilla.dev.extensions/SBGIogdIiwE and
	 * https://developer.mozilla.org/en/XUL_School/Handling_Preferences#Preference_Windows
	 */
	cancel : function() {
		if (EasyCopyPrefsManager.instantApply()) {
			EasyCopyPrefsUI.apply();
		}
	},
	
	saveTemplatesToPrefs : function() {
		var templates = [];
		for (var i = 0; i < EasyCopyPrefsUI.templatesTree.view.rowCount; i++) {
			var item = EasyCopyPrefsUI.templatesTree.view.getItemAtIndex(i);
			var template = item.easyCopyTemplate;
			
			template.order = i;
			
			templates.push(template);
		}
		
		EasyCopyPrefsManager.writePrefsFromTemplate(templates);
	},
	
	deleteTemplatesFromPrefs : function() {
		for (var i = 0; i < EasyCopyPrefsUI.deletedTemplates.length; i++) {
			EasyCopyPrefsManager.deleteTemplate(EasyCopyPrefsUI.deletedTemplates[i]);
		}
	},
	
	moveUp : function() {
		EasyCopyPrefsUI.runOnSelectedItem(function(selectedItem, selectedIndex) {
			var prevItem = selectedItem.previousSibling;
			if (prevItem === null) {
				return;
			}
			
			//Move the selected item up
			var parent = selectedItem.parentNode;
			parent.removeChild(selectedItem);
			parent.insertBefore(selectedItem, prevItem);
			
			EasyCopyPrefsUI.templatesTree.view.selection.select(selectedIndex - 1);
		});
	},
	
	moveDown : function() {
		EasyCopyPrefsUI.runOnSelectedItem(function(selectedItem, selectedIndex) {
			var nextItem = selectedItem.nextSibling;
			if (nextItem === null) {
				return;
			}
			
			//Move the selected item down
			var parent = selectedItem.parentNode;
			parent.removeChild(selectedItem);
			parent.insertBefore(selectedItem, nextItem.nextSibling);
			EasyCopyPrefsUI.templatesTree.view.selection.select(selectedIndex + 1);
		});
	},
	
	newTemplate : function() {
		var treeChildren = EasyCopyPrefsUI.templatesTree.getElementsByTagName("treechildren")[0];
		
		//Build a tree item from the template, insert it into the tree and select it
		var newItem = EasyCopyPrefsUI.buildTemplateItem(EasyCopyTemplate.defaultTemplate());
		var selectedItem = EasyCopyPrefsUI.selectedItem();
		if (selectedItem != null) {
			treeChildren.insertBefore(newItem, selectedItem.nextSibling);
			EasyCopyPrefsUI.templatesTree.view.selection.select(EasyCopyPrefsUI.selectedIndex() + 1);
		} else {
			treeChildren.appendChild(newItem);
			EasyCopyPrefsUI.templatesTree.view.selection.select(0);
		}
		
		var name = document.getElementById("template-name-text");
		name.focus();
		name.select();
		
		EasyCopyPrefsUI.handleNumRowsChanged();
	},
	
	deleteTemplate : function() {
		EasyCopyPrefsUI.runOnSelectedItem(function(selectedItem, selectedIndex) {
			EasyCopyPrefsUI.deletedTemplates.push(selectedItem.easyCopyTemplate);
			selectedItem.parentNode.removeChild(selectedItem);

			//Disable Template details if all templates have been deleted.
			if (EasyCopyPrefsUI.templatesTree.view.rowCount) {
				var rowToSelect = selectedIndex;
				if (rowToSelect >= EasyCopyPrefsUI.templatesTree.view.rowCount) {
					rowToSelect = EasyCopyPrefsUI.templatesTree.view.rowCount - 1;
				}
				EasyCopyPrefsUI.templatesTree.view.selection.select(rowToSelect);
			}
		});
		
		EasyCopyPrefsUI.handleNumRowsChanged();
	},
	
	selectedItem : function() {
		return EasyCopyPrefsUI.templatesTree.view.getItemAtIndex(EasyCopyPrefsUI.templatesTree.view.selection.currentIndex);
	},
	
	selectedIndex : function() {
		return EasyCopyPrefsUI.templatesTree.view.selection.currentIndex;
	},
	

	/**
	 * Runs the function <code>action</code> on the currently selected item, passing in the selectedItem and
	 * selectedIndex. Only runs the action if there is something selected.
	 */
	runOnSelectedItem : function(action) {
		if (EasyCopyPrefsUI.templatesTree.view.selection.count == 0) {
			return;
		}

		var selectedIndex = EasyCopyPrefsUI.selectedIndex();
		var selectedItem = EasyCopyPrefsUI.selectedItem();
		
		action(selectedItem, selectedIndex);
	}
};