var EasyCopyVariableManager = function() {
	EasyCopyVariableManager.DEFAULT_RESOLVER = new EasyCopyVariableResolver();
	
	this.variables = {};

	EasyCopyVariableManager.prototype.registerVariable = function(variable, variableResolver) {
		this.variables[variable.toLowerCase()] = variableResolver;
	},
	
	EasyCopyVariableManager.prototype.variableResolverFor = function(variableName) {
		var variableResolver = this.variables[this.formatVariableName(variableName)];
		if (typeof variableResolver === "undefined" || !this.variables.hasOwnProperty(variableName)) {
			variableResolver = EasyCopyVariableManager.DEFAULT_RESOLVER;
		}

		return variableResolver;
	};
	
	EasyCopyVariableManager.prototype.resolveTemplate = function(template, theDocument) {
		return template.template.replace(this.variableRegEx(), function(variableManager, type) {
			return function(variableWithPercent, variableName) {
				var variableResolver = variableManager.variableResolverFor(variableName);
				var resolvedTemplate = variableResolver.resolve(variableWithPercent, theDocument);
				return variableManager.encodeString(resolvedTemplate, template.formatter);
			};
		} (this, template.type));
	};

	EasyCopyVariableManager.prototype.variableRegEx = function() {
		return /%([a-z_]+)%/gi;
	};

	EasyCopyVariableManager.prototype.formatVariableName = function(variableName) {
		return variableName.toLowerCase();
	};
	
	EasyCopyVariableManager.prototype.encodeString = function(string, formatter) {
		if (formatter == EasyCopyTemplate.HTML_FORMATTER) {
			return string.replace(/&/g, "&amp;")
						.replace(/</g, "&lt;")
						.replace(/>/g, "&gt;");
		}
		
		return string;
	};
	
	EasyCopyVariableManager.prototype.templateAppliesToContext = function(template, theDocument) {
		var variableRegEx = this.variableRegEx();
		var variableName = null;
		while (variableName = variableRegEx.exec(template.template)) {
			var variableResolver = EasyCopy.variableManager.variableResolverFor(variableName[1]);
			if (!variableResolver.appliesToContext(theDocument)) {
				return false;
			}
		}
		
		return true;
	};
};